import { EventSourcePolyfill } from 'event-source-polyfill';
import { debugLog, debugError, debugWarn } from './utils/debug';

/**
 * Configuration options for the SSE client
 */
export interface SSEClientOptions {
  /**
   * Headers to include in the SSE request
   */
  headers?: Record<string, string>;

  /**
   * Whether to include credentials in the request
   * @default true
   */
  withCredentials?: boolean;

  /**
   * Timeout for heartbeat in milliseconds
   * @default 21600000 (6 hours)
   */
  heartbeatTimeout?: number;

  /**
   * Maximum number of retry attempts
   * @default 10
   */
  maxRetryAttempts?: number;

  /**
   * Initial delay for retry in milliseconds
   * @default 1000
   */
  initialRetryDelay?: number;

  /**
   * Maximum delay for retry in milliseconds
   * @default 30000
   */
  maxRetryDelay?: number;

  /**
   * Whether to automatically reconnect on error
   * @default true
   */
  autoReconnect?: boolean;

  /**
   * Debug mode
   * @default false
   */
  debug?: boolean;

  /**
   * Whether to bypass SSL certificate validation
   * Only works in Node.js environments
   * @default false
   */
  skipSSLValidation?: boolean;
}

/**
 * Default options for the SSE client
 */
const DEFAULT_OPTIONS: Required<Omit<SSEClientOptions, 'headers'>> = {
  withCredentials: true,
  heartbeatTimeout: 21600000, // 6 hours
  maxRetryAttempts: 10,
  initialRetryDelay: 1000,
  maxRetryDelay: 30000,
  autoReconnect: true,
  debug: false,
  skipSSLValidation: false
};

/**
 * Helper: delay for ms
 */
function delay(ms: number): Promise<void> {
  return new Promise((resolve) => setTimeout(resolve, ms));
}

/**
 * A custom SSE client that supports headers and bypasses certificate issues
 */
export class SSEClient {
  private eventSource: EventSourcePolyfill | null = null;
  private url: string;
  private options: Required<SSEClientOptions>;
  private retryCount = 0;
  private closeRequested = false;
  private eventListeners: Map<string, Set<(event: any) => void>> = new Map();
  private customEventTarget: EventTarget;

  /**
   * Creates a new SSE client
   * @param url The URL to connect to
   * @param options Configuration options
   */
  constructor(url: string, options: SSEClientOptions = {}) {
    this.url = url;
    this.options = {
      ...DEFAULT_OPTIONS,
      headers: options.headers || {},
      ...options
    };

    // Set debug mode
    if (this.options.debug) {
      (globalThis as any).__SSE_DEBUG_ENABLED = true;
    }

    // Create a custom event target for dispatching events
    this.customEventTarget = typeof window !== 'undefined'
      ? document.createElement('div')
      : new EventTarget();

    debugLog('SSEClient: Created new client for URL:', url);
  }

  /**
   * Connects to the SSE endpoint
   * @returns The SSE client instance for chaining
   */
  public connect(): SSEClient {
    if (this.eventSource) {
      debugWarn('SSEClient: Already connected, closing existing connection');
      this.close();
    }

    this.closeRequested = false;
    this.createConnection();
    return this;
  }

  /**
   * Closes the SSE connection
   */
  public close(): void {
    this.closeRequested = true;
    if (this.eventSource) {
      debugLog('SSEClient: Closing connection');
      this.eventSource.close();
      this.eventSource = null;
    }
  }

  /**
   * Adds an event listener for SSE events
   * @param eventName The name of the event to listen for
   * @param listener The event listener function
   * @returns The SSE client instance for chaining
   */
  public on(eventName: string, listener: (event: any) => void): SSEClient {
    if (!this.eventListeners.has(eventName)) {
      this.eventListeners.set(eventName, new Set());
    }

    this.eventListeners.get(eventName)!.add(listener);

    // If already connected, add the listener to the event source
    if (this.eventSource) {
      this.addEventSourceListener(eventName, listener);
    }

    return this;
  }

  /**
   * Removes an event listener
   * @param eventName The name of the event
   * @param listener The event listener function to remove
   * @returns The SSE client instance for chaining
   */
  public off(eventName: string, listener: (event: any) => void): SSEClient {
    const listeners = this.eventListeners.get(eventName);
    if (listeners) {
      listeners.delete(listener);
      if (listeners.size === 0) {
        this.eventListeners.delete(eventName);
      }
    }

    return this;
  }

  /**
   * Removes all event listeners for a specific event
   * @param eventName The name of the event
   * @returns The SSE client instance for chaining
   */
  public removeAllListeners(eventName?: string): SSEClient {
    if (eventName) {
      this.eventListeners.delete(eventName);
    } else {
      this.eventListeners.clear();
    }

    return this;
  }

  /**
   * Creates a new SSE connection
   */
  private createConnection(): void {
    debugLog('SSEClient: Creating new connection to:', this.url);

    try {
      // Create connection options
      const connectionOptions: any = {
        headers: this.options.headers,
        withCredentials: this.options.withCredentials,
        heartbeatTimeout: this.options.heartbeatTimeout
      };

      // Add SSL bypass if requested and in Node.js environment
      if (this.options.skipSSLValidation && typeof window === 'undefined') {
        try {
          // Only import https in Node.js environment
          const https = require('https');
          connectionOptions.httpsAgent = new https.Agent({ rejectUnauthorized: false });
          debugLog('SSEClient: SSL certificate validation bypassed');
        } catch (e) {
          debugWarn('SSEClient: Failed to create https agent for SSL bypass:', e);
        }
      }

      this.eventSource = new EventSourcePolyfill(this.url, connectionOptions);

      // Set up event handlers
      this.eventSource.onopen = this.handleOpen.bind(this);
      this.eventSource.onerror = this.handleError.bind(this);

      // Add event listeners for all registered events
      for (const [eventName, listeners] of this.eventListeners.entries()) {
        for (const listener of listeners) {
          this.addEventSourceListener(eventName, listener);
        }
      }
    } catch (error) {
      debugError('SSEClient: Error creating connection:', error);
      this.handleError(error);
    }
  }

  /**
   * Adds an event listener to the event source
   * @param eventName The name of the event
   * @param listener The event listener function
   */
  private addEventSourceListener(eventName: string, listener: (event: any) => void): void {
    if (!this.eventSource) return;

    this.eventSource.addEventListener(eventName, (event: any) => {
      try {
        debugLog(`SSEClient: Received ${eventName} event:`, event);

        // Parse the event data if it's a string
        if (typeof event.data === 'string') {
          try {
            const parsedData = JSON.parse(event.data);
            event.parsedData = parsedData;
          } catch (e) {
            // If parsing fails, just use the raw data
            debugWarn(`SSEClient: Failed to parse event data as JSON:`, e);
          }
        }

        // Call the listener with the event
        listener(event);
      } catch (error) {
        debugError(`SSEClient: Error handling ${eventName} event:`, error);
      }
    });
  }

  /**
   * Handles the open event
   */
  private handleOpen(): void {
    debugLog('SSEClient: Connection opened');
    this.retryCount = 0;

    // Dispatch the open event
    const openEvent = new Event('open');
    this.customEventTarget.dispatchEvent(openEvent);
  }

  /**
   * Handles the error event
   * @param error The error that occurred
   */
  private handleError(error: any): void {
    debugError('SSEClient: Connection error:', error);

    // Dispatch the error event
    const errorEvent = new CustomEvent('error', { detail: error });
    this.customEventTarget.dispatchEvent(errorEvent);

    // If close was requested, don't reconnect
    if (this.closeRequested) {
      debugLog('SSEClient: Not reconnecting (close requested)');
      return;
    }

    // If auto reconnect is disabled, don't reconnect
    if (!this.options.autoReconnect) {
      debugLog('SSEClient: Not reconnecting (auto reconnect disabled)');
      return;
    }

    // If max retry attempts reached, don't reconnect
    if (this.retryCount >= this.options.maxRetryAttempts) {
      debugError(`SSEClient: Max retry attempts (${this.options.maxRetryAttempts}) reached, not reconnecting`);
      return;
    }

    // Reconnect with exponential backoff
    this.reconnect();
  }

  /**
   * Reconnects to the SSE endpoint with exponential backoff
   */
  private async reconnect(): Promise<void> {
    this.retryCount++;

    // Calculate delay with exponential backoff
    const delayMs = Math.min(
      this.options.initialRetryDelay * Math.pow(2, this.retryCount - 1),
      this.options.maxRetryDelay
    );

    debugLog(`SSEClient: Reconnecting in ${delayMs}ms (attempt ${this.retryCount}/${this.options.maxRetryAttempts})`);

    // Wait for the delay
    await delay(delayMs);

    // If close was requested during the delay, don't reconnect
    if (this.closeRequested) {
      debugLog('SSEClient: Not reconnecting (close requested during delay)');
      return;
    }

    // Create a new connection
    this.createConnection();
  }
}

/**
 * Singleton pattern for SSE connections
 */
export class SSEConnectionManager {
  private static instance: SSEConnectionManager;
  private connections: Map<string, SSEClient> = new Map();

  private constructor() {}

  /**
   * Gets the singleton instance
   * @returns The SSE connection manager instance
   */
  public static getInstance(): SSEConnectionManager {
    if (!SSEConnectionManager.instance) {
      SSEConnectionManager.instance = new SSEConnectionManager();
    }
    return SSEConnectionManager.instance;
  }

  /**
   * Gets or creates a connection for the given ID
   * @param url The URL to connect to
   * @param id A unique identifier for the connection
   * @param options Configuration options
   * @returns The SSE client instance
   */
  public getConnection(url: string, id: string, options: SSEClientOptions = {}): SSEClient {
    if (this.connections.has(id)) {
      debugLog(`SSEConnectionManager: Reusing existing connection for ID ${id}`);
      return this.connections.get(id)!;
    }

    debugLog(`SSEConnectionManager: Creating new connection for ID ${id}`);
    const client = new SSEClient(url, options).connect();
    this.connections.set(id, client);
    return client;
  }

  /**
   * Closes a connection for the given ID
   * @param id The connection ID
   */
  public closeConnection(id: string): void {
    if (this.connections.has(id)) {
      debugLog(`SSEConnectionManager: Closing connection for ID ${id}`);
      this.connections.get(id)!.close();
      this.connections.delete(id);
    }
  }

  /**
   * Closes all connections
   */
  public closeAllConnections(): void {
    debugLog(`SSEConnectionManager: Closing all connections`);
    for (const [id, client] of this.connections.entries()) {
      client.close();
      this.connections.delete(id);
    }
  }
}

/**
 * Gets or creates a connection for the given ID
 * @param url The URL to connect to
 * @param id A unique identifier for the connection
 * @param options Configuration options
 * @returns The SSE client instance
 */
export function getSSEConnection(url: string, id: string, options: SSEClientOptions = {}): SSEClient {
  return SSEConnectionManager.getInstance().getConnection(url, id, options);
}

/**
 * Closes a connection for the given ID
 * @param id The connection ID
 */
export function closeSSEConnection(id: string): void {
  SSEConnectionManager.getInstance().closeConnection(id);
}

/**
 * Closes all connections
 */
export function closeAllSSEConnections(): void {
  SSEConnectionManager.getInstance().closeAllConnections();
}

/**
 * Sets up event listeners to clean up SSE connections when the page is unloaded
 */
export function setupSSECleanup(): void {
  if (typeof window === 'undefined') return;

  // Clean up all SSE connections when the page is unloaded
  window.addEventListener('beforeunload', () => {
    debugLog('SSECleanup: Cleaning up all SSE connections before page unload');
    closeAllSSEConnections();
  });
}
